@@ -2,7 +2,10 @@ use std::borrow::Borrow;
 use std::collections::HashMap;
 use std::rc::Rc;
 
-use crate::compiler::comptypes::{Binding, BodyForm, CompileForm, HelperForm, LetFormKind};
+use crate::compiler::comptypes::{
+    Binding, BodyForm, CompileForm, DefconstData, DefmacData, DefunData, HelperForm, LetData,
+    LetFormKind,
+};
 use crate::compiler::gensym::gensym;
 use crate::compiler::sexp::SExp;
 
@@ -106,6 +109,7 @@ fn make_binding_unique(b: &Binding) -> (Vec<u8>, Binding) {
         b.name.to_vec(),
         Binding {
             loc: b.loc.clone(),
+            nl: b.nl.clone(),
             name: gensym(b.name.clone()),
             body: b.body.clone(),
         },
@@ -114,19 +118,29 @@ fn make_binding_unique(b: &Binding) -> (Vec<u8>, Binding) {
 
 fn rename_in_bodyform(namemap: &HashMap<Vec<u8>, Vec<u8>>, b: Rc<BodyForm>) -> BodyForm {
     match b.borrow() {
-        BodyForm::Let(l, kind, bindings, body) => {
-            let new_bindings = bindings
+        BodyForm::Let(kind, letdata) => {
+            let new_bindings = letdata
+                .bindings
                 .iter()
                 .map(|b| {
                     Rc::new(Binding {
                         loc: b.loc(),
+                        nl: b.nl.clone(),
                         name: b.name.clone(),
                         body: Rc::new(rename_in_bodyform(namemap, b.body.clone())),
                     })
                 })
                 .collect();
-            let new_body = rename_in_bodyform(namemap, body.clone());
-            BodyForm::Let(l.clone(), kind.clone(), new_bindings, Rc::new(new_body))
+            let new_body = rename_in_bodyform(namemap, letdata.body.clone());
+            BodyForm::Let(
+                kind.clone(),
+                LetData {
+                    loc: letdata.loc.clone(),
+                    kw: letdata.kw.clone(),
+                    bindings: new_bindings,
+                    body: Rc::new(new_body),
+                },
+            )
         }
 
         BodyForm::Quoted(atom) => match atom.borrow() {
@@ -152,6 +166,8 @@ fn rename_in_bodyform(namemap: &HashMap<Vec<u8>, Vec<u8>>, b: Rc<BodyForm>) -> B
                 .collect();
             BodyForm::Call(l.clone(), new_vs)
         }
+
+        BodyForm::Mod(l, prog) => BodyForm::Mod(l.clone(), prog.clone()),
     }
 }
 
@@ -167,10 +183,13 @@ pub fn desugar_sequential_let_bindings(
         desugar_sequential_let_bindings(
             bindings,
             &BodyForm::Let(
-                want_binding.loc(),
                 LetFormKind::Parallel,
-                vec![want_binding],
-                Rc::new(body.clone()),
+                LetData {
+                    loc: want_binding.loc(),
+                    kw: None,
+                    bindings: vec![want_binding],
+                    body: Rc::new(body.clone()),
+                },
             ),
             n - 1,
         )
@@ -179,18 +198,19 @@ pub fn desugar_sequential_let_bindings(
 
 fn rename_args_bodyform(b: &BodyForm) -> BodyForm {
     match b.borrow() {
-        BodyForm::Let(_l, LetFormKind::Sequential, bindings, body) => {
+        BodyForm::Let(LetFormKind::Sequential, letdata) => {
             // Renaming a sequential let is exactly as if the bindings were
             // nested in separate parallel lets.
             rename_args_bodyform(&desugar_sequential_let_bindings(
-                bindings,
-                body,
-                bindings.len(),
+                &letdata.bindings,
+                letdata.body.borrow(),
+                letdata.bindings.len(),
             ))
         }
 
-        BodyForm::Let(l, LetFormKind::Parallel, bindings, body) => {
-            let renames: Vec<(Vec<u8>, Binding)> = bindings
+        BodyForm::Let(LetFormKind::Parallel, letdata) => {
+            let renames: Vec<(Vec<u8>, Binding)> = letdata
+                .bindings
                 .iter()
                 .map(|x| make_binding_unique(x.borrow()))
                 .collect();
@@ -206,17 +226,21 @@ fn rename_args_bodyform(b: &BodyForm) -> BodyForm {
                 .map(|x| {
                     Rc::new(Binding {
                         loc: x.loc.clone(),
+                        nl: x.nl.clone(),
                         name: x.name.clone(),
                         body: Rc::new(rename_args_bodyform(&x.body)),
                     })
                 })
                 .collect();
-            let locally_renamed_body = rename_in_bodyform(&local_namemap, body.clone());
+            let locally_renamed_body = rename_in_bodyform(&local_namemap, letdata.body.clone());
             BodyForm::Let(
-                l.clone(),
                 LetFormKind::Parallel,
-                new_bindings,
-                Rc::new(locally_renamed_body),
+                LetData {
+                    loc: letdata.loc.clone(),
+                    kw: letdata.kw.clone(),
+                    bindings: new_bindings,
+                    body: Rc::new(locally_renamed_body),
+                },
             )
         }
 
@@ -230,75 +254,94 @@ fn rename_args_bodyform(b: &BodyForm) -> BodyForm {
                 .collect();
             BodyForm::Call(l.clone(), new_vs)
         }
+        BodyForm::Mod(l, program) => BodyForm::Mod(l.clone(), program.clone()),
     }
 }
 
 fn rename_in_helperform(namemap: &HashMap<Vec<u8>, Vec<u8>>, h: &HelperForm) -> HelperForm {
     match h {
-        HelperForm::Defconstant(l, n, body) => HelperForm::Defconstant(
-            l.clone(),
-            n.to_vec(),
-            Rc::new(rename_in_bodyform(namemap, body.clone())),
-        ),
-        HelperForm::Defmacro(l, n, arg, body) => HelperForm::Defmacro(
-            l.clone(),
-            n.to_vec(),
-            arg.clone(),
-            Rc::new(rename_in_compileform(namemap, body.clone())),
-        ),
-        HelperForm::Defun(l, n, inline, arg, body) => HelperForm::Defun(
-            l.clone(),
-            n.to_vec(),
+        HelperForm::Defconstant(defc) => HelperForm::Defconstant(DefconstData {
+            loc: defc.loc.clone(),
+            name: defc.name.to_vec(),
+            nl: defc.nl.clone(),
+            kw: defc.kw.clone(),
+            body: Rc::new(rename_in_bodyform(namemap, defc.body.clone())),
+        }),
+        HelperForm::Defmacro(mac) => HelperForm::Defmacro(DefmacData {
+            loc: mac.loc.clone(),
+            kw: mac.kw.clone(),
+            nl: mac.nl.clone(),
+            name: mac.name.to_vec(),
+            args: mac.args.clone(),
+            program: Rc::new(rename_in_compileform(namemap, mac.program.clone())),
+        }),
+        HelperForm::Defun(inline, defun) => HelperForm::Defun(
             *inline,
-            arg.clone(),
-            Rc::new(rename_in_bodyform(namemap, body.clone())),
+            DefunData {
+                loc: defun.loc.clone(),
+                kw: defun.kw.clone(),
+                nl: defun.nl.clone(),
+                name: defun.name.to_vec(),
+                args: defun.args.clone(),
+                body: Rc::new(rename_in_bodyform(namemap, defun.body.clone())),
+            },
         ),
     }
 }
 
 fn rename_args_helperform(h: &HelperForm) -> HelperForm {
     match h {
-        HelperForm::Defconstant(l, n, body) => {
-            HelperForm::Defconstant(l.clone(), n.clone(), Rc::new(rename_args_bodyform(body)))
-        }
-        HelperForm::Defmacro(l, n, arg, body) => {
+        HelperForm::Defconstant(defc) => HelperForm::Defconstant(DefconstData {
+            loc: defc.loc.clone(),
+            nl: defc.nl.clone(),
+            kw: defc.kw.clone(),
+            name: defc.name.clone(),
+            body: Rc::new(rename_args_bodyform(defc.body.borrow())),
+        }),
+        HelperForm::Defmacro(mac) => {
             let mut new_names: HashMap<Vec<u8>, Vec<u8>> = HashMap::new();
-            for x in invent_new_names_sexp(arg.clone()).iter() {
+            for x in invent_new_names_sexp(mac.args.clone()).iter() {
                 new_names.insert(x.0.clone(), x.1.clone());
             }
             let mut local_namemap = HashMap::new();
             for x in new_names.iter() {
                 local_namemap.insert(x.0.to_vec(), x.1.to_vec());
             }
-            let local_renamed_arg = rename_in_cons(&local_namemap, arg.clone());
-            let local_renamed_body = rename_args_compileform(body);
-            HelperForm::Defmacro(
-                l.clone(),
-                n.clone(),
-                local_renamed_arg,
-                Rc::new(rename_in_compileform(
+            let local_renamed_arg = rename_in_cons(&local_namemap, mac.args.clone());
+            let local_renamed_body = rename_args_compileform(mac.program.borrow());
+            HelperForm::Defmacro(DefmacData {
+                loc: mac.loc.clone(),
+                kw: mac.kw.clone(),
+                nl: mac.nl.clone(),
+                name: mac.name.clone(),
+                args: local_renamed_arg,
+                program: Rc::new(rename_in_compileform(
                     &local_namemap,
                     Rc::new(local_renamed_body),
                 )),
-            )
+            })
         }
-        HelperForm::Defun(l, n, inline, arg, body) => {
-            let new_names = invent_new_names_sexp(arg.clone());
+        HelperForm::Defun(inline, defun) => {
+            let new_names = invent_new_names_sexp(defun.args.clone());
             let mut local_namemap = HashMap::new();
             for x in new_names.iter() {
                 local_namemap.insert(x.0.clone(), x.1.clone());
             }
-            let local_renamed_arg = rename_in_cons(&local_namemap, arg.clone());
-            let local_renamed_body = rename_args_bodyform(body);
+            let local_renamed_arg = rename_in_cons(&local_namemap, defun.args.clone());
+            let local_renamed_body = rename_args_bodyform(defun.body.borrow());
             HelperForm::Defun(
-                l.clone(),
-                n.clone(),
                 *inline,
-                local_renamed_arg,
-                Rc::new(rename_in_bodyform(
-                    &local_namemap,
-                    Rc::new(local_renamed_body),
-                )),
+                DefunData {
+                    loc: defun.loc.clone(),
+                    nl: defun.nl.clone(),
+                    kw: defun.kw.clone(),
+                    name: defun.name.clone(),
+                    args: local_renamed_arg,
+                    body: Rc::new(rename_in_bodyform(
+                        &local_namemap,
+                        Rc::new(local_renamed_body),
+                    )),
+                },
             )
         }
     }
@@ -308,6 +351,7 @@ fn rename_in_compileform(namemap: &HashMap<Vec<u8>, Vec<u8>>, c: Rc<CompileForm>
     CompileForm {
         loc: c.loc.clone(),
         args: c.args.clone(),
+        include_forms: c.include_forms.clone(),
         helpers: c
             .helpers
             .iter()
@@ -323,6 +367,7 @@ pub fn rename_children_compileform(c: &CompileForm) -> CompileForm {
     CompileForm {
         loc: c.loc.clone(),
         args: c.args.clone(),
+        include_forms: c.include_forms.clone(),
         helpers: local_renamed_helpers,
         exp: Rc::new(local_renamed_body),
     }
@@ -341,6 +386,7 @@ pub fn rename_args_compileform(c: &CompileForm) -> CompileForm {
     CompileForm {
         loc: c.loc(),
         args: local_renamed_arg,
+        include_forms: c.include_forms.clone(),
         helpers: local_renamed_helpers
             .iter()
             .map(|x| rename_in_helperform(&local_namemap, x))
