@@ -17,7 +17,7 @@ fn test_cons_q_a(src: String) -> String {
     let memo = RefCell::new(HashMap::new());
     let input_ir = read_ir(&src).unwrap();
     let assembled = assemble_from_ir(&mut allocator, Rc::new(input_ir)).unwrap();
-    let runner = run_program_for_search_paths(&vec![".".to_string()]);
+    let runner = run_program_for_search_paths("*test*", &vec![".".to_string()], false);
     let optimized = cons_q_a_optimizer(&mut allocator, &memo, assembled, runner.clone()).unwrap();
     disassemble(&mut allocator, optimized)
 }
@@ -27,7 +27,7 @@ fn test_children_optimizer(src: String) -> String {
     let memo = RefCell::new(HashMap::new());
     let input_ir = read_ir(&src).unwrap();
     let assembled = assemble_from_ir(&mut allocator, Rc::new(input_ir)).unwrap();
-    let runner = run_program_for_search_paths(&vec![".".to_string()]);
+    let runner = run_program_for_search_paths("*test*", &vec![".".to_string()], false);
     let optimized = children_optimizer(&mut allocator, &memo, assembled, runner.clone()).unwrap();
     disassemble(&mut allocator, optimized)
 }
@@ -37,7 +37,7 @@ fn test_constant_optimizer(src: String) -> String {
     let memo = RefCell::new(HashMap::new());
     let input_ir = read_ir(&src).unwrap();
     let assembled = assemble_from_ir(&mut allocator, Rc::new(input_ir)).unwrap();
-    let runner = run_program_for_search_paths(&vec![".".to_string()]);
+    let runner = run_program_for_search_paths("*test*", &vec![".".to_string()], false);
     let optimized =
         constant_optimizer(&mut allocator, &memo, assembled, 0, runner.clone()).unwrap();
     disassemble(&mut allocator, optimized)
@@ -47,7 +47,7 @@ fn test_optimizer(src: String) -> String {
     let mut allocator = Allocator::new();
     let input_ir = read_ir(&src).unwrap();
     let assembled = assemble_from_ir(&mut allocator, Rc::new(input_ir)).unwrap();
-    let runner = run_program_for_search_paths(&vec![".".to_string()]);
+    let runner = run_program_for_search_paths("*test*", &vec![".".to_string()], false);
     let optimized = optimize_sexp(&mut allocator, assembled, runner.clone()).unwrap();
     disassemble(&mut allocator, optimized)
 }
