@@ -176,3 +176,13 @@ fn test_collatz() {
         "(q . 7)"
     );
 }
+
+#[test]
+fn test_mod_in_repl() {
+    assert_eq!(
+        test_repl_outcome(vec!["(a (mod (X) (+ 1 (* 3 X))) (list 3))"])
+            .unwrap()
+            .unwrap(),
+        "(q . 10)"
+    );
+}
